@@ -32,13 +32,15 @@ class ProfileJSONDecoder(json.JSONDecoder):
 
 
 class Profile(object):
-    def __init__(self, config, cache_size=4*1024*1024):
+    def __init__(self, config, cache_size=4*1024*1024, write_availability=True):
         self.config = ccbrowse.config.default_config
         self.config.update(config)
+        self._write_availability = write_availability
 
         self.storage = ccbrowse.storage.Router(
             self.config['storage'],
             root=self.config['root'],
+            profile=self,
             on_store=lambda obj: self.serialize(obj),
             on_retrieve=lambda obj: self.deserialize(obj),
         )
@@ -60,7 +62,8 @@ class Profile(object):
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        self.write_availability()
+        if self._write_availability:
+            self.write_availability()
         self.cache.empty()
 
     def __contains__(self, item):
@@ -86,6 +89,12 @@ class Profile(object):
             raise ValueError('No such layer %s' % name)
         return self['layers'][name]
 
+    def layer_product(self, layer):
+        for name, cls in PRODUCTS.items():
+            if layer in cls.datasets(self['primary'] == name):
+                return name
+        raise ValueError('No product for layer %s' % layer)
+
     def deserialize(self, obj):
         """Deserialize obj.raw_data to obj.data by a format-specific method."""
         if not ('raw_data' in obj or 'data' in obj):
@@ -133,7 +142,7 @@ class Profile(object):
                 cls = PRODUCTS[ref['product']]
             except KeyError:
                 raise RuntimeError('Unknown product type "%s"' % ref['product'])
-            product = cls(ref['filename'], self)
+            product = cls(ref['filename'], self, ref['offset'])
             tile = product.tile(obj['layer'], obj['zoom'], obj['x'], obj['z'])
             if 'data' in obj:
                 if obj['format'] == 'png':
@@ -150,7 +159,7 @@ class Profile(object):
 
         Object is a dictionary with the following mandatory and optional fields:
 
-            layer   layer name [required]
+            layer   layer name
             data    numpy array (format: png) or dictionary (format: json)
             ref     list of references to product files
             zoom    zoom level
@@ -161,12 +170,13 @@ class Profile(object):
         Return the object augmented with layer fields.
         """
         # Insert layer properties to obj.
-        layer = self.layer_for(obj)
-        o = layer.copy()
-        o.update(obj)
-        obj = o
+        if 'layer' in obj:
+            layer = self.layer_for(obj)
+            o = layer.copy()
+            o.update(obj)
+            obj = o
 
-        if append:
+        if append and 'layer' in obj:
             orig_obj = self.load(obj, dereference=False)
             if orig_obj is not None:
                 # Update data.
@@ -189,20 +199,27 @@ class Profile(object):
                 del obj['raw_data']
 
         #self.cache.store(obj)
-        self.storage.store(obj)
+        res = self.storage.store(obj)
 
         if 'zoom' in obj and 'x' in obj:
-            self.update_availability(obj['layer'], obj['zoom'], (obj['x'], obj['x']+1))
+            self.update_availability(
+                obj['layer'],
+                obj['zoom'],
+                [(obj['x'], obj['x']+1)],
+            )
 
-        return obj
+        return res
 
     def load(self, obj, exclude=[], dereference=True):
         """Load object from profile."""
         try: layer = self.layer_for(obj)
         except ValueError: return None
 
+        product = self.layer_product(obj['layer'])
+
         o = layer.copy()
         o.update(obj)
+        o['product'] = product
         obj = o
         if 'data' in obj: del obj['data']
 
@@ -212,7 +229,7 @@ class Profile(object):
             if o != None:
                 if dereference and 'ref' in o:
                     self.dereference(o)
-                    self.save(o, append=False)
+                    #self.save(o, append=False)
                 o2 = layer.copy()
                 o2.update(o)
                 self.cache.store(o2, dirty=False)
@@ -229,7 +246,10 @@ class Profile(object):
             colormap = json.load(fp)
         return colormap
 
-    def get_availability(self, layer):
+    def get_availability(self, layer=None):
+        if layer is None:
+            return self.availability
+
         if layer in self.availability:
             return self.availability[layer]
 
@@ -272,10 +292,9 @@ class Profile(object):
             }
             self.storage.store(obj)
 
-    def update_availability(self, layer, level, xxx_todo_changeme):
-        (start, stop) = xxx_todo_changeme
+    def update_availability(self, layer, level, intervals):
         availability = self.get_availability(layer)
-        if level in availability:
+        if level not in availability:
+            availability[level] = RangeList()
+        for start, stop in intervals:
             availability[level].append(start, stop)
-        else:
-            availability[level] = RangeList([(start, stop)])
