@@ -5,6 +5,7 @@ from .sqlite import *
 from .memcache import *
 from .htree import *
 from .s3 import *
+from .fileref import *
 
 DRIVERS = {
     'null': NullDriver,
@@ -13,6 +14,7 @@ DRIVERS = {
     'memcache': MemCacheDriver,
     'htree': HtreeDriver,
     's3': S3Driver,
+    'fileref': FilerefDriver,
 }
 
 from .router import *
