@@ -2,7 +2,7 @@ from .driver import Driver
 
 class NullDriver(Driver):
     def store(self, obj):
-        pass
-        
+        return False
+
     def retrieve(self, obj, exclude=[]):
         return None
