@@ -40,9 +40,9 @@ def substitute(s, variables):
     variables. Substitution is performed by eval(var, variables).
     Unrecognized variables are replaced by an empty string.
     """
-    params = []
+    env = variables.copy()
     def repl(m):
-        try: return str(eval(m.group(1), variables))
+        try: return str(eval(m.group(1), env))
         except: return ''
     return re.sub('\{(.+?)\}', repl, s)
 
@@ -335,3 +335,17 @@ class JSONEncoder(json.JSONEncoder):
         if isinstance(obj, np.ndarray):
             return obj.tolist()
         return JSONEncoder.default(self, obj)
+
+
+def intervals(x):
+    if len(x) == 0: return []
+    out = []
+    diff = np.diff(x)
+    ii = np.where(diff > 1)[0]
+    b = -1
+    for k, i in enumerate(ii):
+        a = ii[k-1] + 1 if k > 0 else 0
+        b = i
+        out += [[x[a], x[b] + 1]]
+    out += [[x[b + 1], x[-1] + 1]]
+    return out
