@@ -12,13 +12,15 @@ L.Projection.Custom = {
 
     unproject: function (point, unbounded) {
         return new L.LatLng(point.y, point.x, true);
-    }
+    },
+
+    bounds: new L.bounds([-Infinity, Infinity], [-Infinity, Infinity])
 };
 
 L.CRS.Custom = function(profile) {
     var transformation = new L.Transformation(
         1/profile.zoom[0].width, 0,
-        -1/profile.zoom[0].height, 1
+        -1/profile.zoom[0].height, 0
     );
     var scale = function(zoom) {
         return 256*profile.zoom[0].width/profile.zoom[zoom].width;
@@ -27,6 +29,9 @@ L.CRS.Custom = function(profile) {
         code: 'EPSG:0000',
         projection: L.Projection.Custom,
         transformation: transformation,
-        scale: scale
+        scale: scale,
+        wrapLng: undefined,
+        wrapLat: undefined,
+        infinite: true
     });
 };
